//  - 
//_________________________________________________________________________
{
Project: usbhidioc.cpp
Version: 1.2
Date: 8/17/00
by Jan Axelson (jan@lvr.com)

Purpose: demonstrates USB communications with a HID-class device

This application was created with Visual C++ 6's AppWizard as a dialog-based application.

Companion firmware and other sample code is available from www.lvr.com.

I owe a big thank you to the many generous contributors on Usenet and the excellent archive at 
www.deja.com/usenet for helping me with Visual C++ in this project. Other sources I used were 
www.natsemi.com's example HID application and John Hyde's terminal program at 
www.usb-by-example.com.
}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    btSearch: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    cbVendorId: TComboBox;
    cbProductId: TComboBox;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    cbReportId: TComboBox;
    cbByte0: TComboBox;
    cbByte1: TComboBox;
    btTransmit: TButton;
    btReceiver: TButton;
    GroupBox3: TGroupBox;
    lbLog: TListBox;
    GroupBox4: TGroupBox;
    lbBytesReceived: TListBox;
    Timer1: TTimer;
    cbAuto: TCheckBox;
    procedure btSearchClick(Sender: TObject);
    procedure btTransmitClick(Sender: TObject);
    procedure btReceiverClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbReportIdChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure cbAutoClick(Sender: TObject);
  private
    { Private declarations }
    procedure WMDEVICECHANGE(var HMessage: TMessage); message WM_DEVICECHANGE;
  public
    { Public declarations }
    //    HID
    function FindTheHID(): LongBool;
    procedure GetDeviceCapabilities();
    procedure PrepareForOverlappedTransfer();
    procedure ReadReport();
    procedure WriteReport();
    //  
    procedure DisplayInputReport();
    procedure DisplayCurrentTime();
    procedure DisplayData(cstrDataToDisplay: String);
    procedure DisplayLastError(Operation: String);
    procedure DisplayReceivedData(ReceivedByte: Byte);

    function GetSerialNumber(HidReadHandle: Cardinal): WideString;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

Uses SetupApi, Hid;

//  
var

  ActualBytesRead: DWORD;
  BytesRead: DWORD;
  Capabilities: _HIDP_CAPS;
  cbBytesRead: DWORD;
  detailData: PSPDeviceInterfaceDetailData;
  DeviceDetected: LongBool;
  DeviceHandle: THandle;
  dwError: DWORD;
  hEventObject: THandle;
  PDevInfo: PHandle;
  HidGuid: TGuid;
  HIDOverlapped: OverLapped;
  InputReport: array [0..4] of Byte;
  Length: Cardinal;
  lpOverLap: POverLapped;
  NumberOfBytesRead: DWORD;
  ReadHandle: Cardinal;//THandle;
  Required: Cardinal;
  ValueToDisplay: String;

//     
procedure TForm1.WMDEVICECHANGE(var HMessage: TMessage);
begin
  if (HMessage.Msg = WM_DEVICECHANGE) then
    FindTheHID();
end;

function TForm1.FindTheHID(): LongBool;
var
  Attributes: THIDDAttributes;
  devInfoData: TSPDeviceInterfaceData;
  LastDevice: LongBool;
  MemberIndex: DWORD;
  MyDeviceDetected: LongBool;
  Resultat: LongBool;
begin
  Length := 0;
	detailData := nil;
	DeviceHandle := 0;

  //  GUID   HID
	HidD_GetHidGuid(HidGuid);
  //   PnP  HID-
  PDevInfo := SetupDiGetClassDevs(@HidGuid,
		                            nil,
                                0,
                                DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);
  //  ,  
  Result := FALSE;
  If PDevInfo = Pointer(INVALID_HANDLE_VALUE) then Exit;

	devInfoData.cbSize := SizeOf(devInfoData);
  //    
  MemberIndex := 0;
	LastDevice := FALSE;

  repeat
    MyDeviceDetected := FALSE;
		//       MemberIndex
		Resultat := SetupDiEnumDeviceInterfaces(PDevInfo,
                                        nil,
                                        HidGuid,
                                        MemberIndex,
                                        devInfoData);

		if Resultat <> False then
    begin
			//       devInfoData
      //      ,   
      // ,   ,   
			Resultat := SetupDiGetDeviceInterfaceDetail(PDevInfo,
				                                          @devInfoData,
                                                  nil,
				                                          0,
                                                  @Length,
                                                  nil);

			// 
			detailData := AllocMem(Length);

			detailData.cbSize := (sizeof(PSPDeviceInterfaceDetailData)) + 1;

			//  ,   .
			Resultat := SetupDiGetDeviceInterfaceDetail(PDevInfo,
				                                          @devInfoData,
				                                          detailData,
				                                          Length,
				                                          @Required,
				                                          nil);

			DeviceHandle := CreateFile(detailData.DevicePath,
				                          GENERIC_READ or GENERIC_WRITE,
				                          FILE_SHARE_READ or FILE_SHARE_WRITE,
				                          nil,
				                          OPEN_EXISTING,
				                          0,
				                          0);

			DisplayLastError('CreateFile: ');

			//   
      Attributes.Size := SizeOf(THIDDAttributes);

			Resultat := HidD_GetAttributes(DeviceHandle, Attributes);

			DisplayLastError('HidD_GetAttributes: ');

			//   
			MyDeviceDetected := FALSE;

			if (Attributes.VendorID = StrToInt(cbVendorId.Text)) then
      begin
				if (Attributes.ProductID = StrToInt(cbProductID.Text)) then
        begin
					// 
					MyDeviceDetected := TRUE;
					DisplayData('Device detected');
					//Get the device's capablities.
					GetDeviceCapabilities();
    			PrepareForOverlappedTransfer();
				end //if (Attributes.ProductID == ProductID)
				else
					//  .
					CloseHandle(DeviceHandle);
      end
			else begin
				//  .
				CloseHandle(DeviceHandle);
        DeviceHandle := 0;
		    //Free the memory used by the detailData structure (no longer needed).
		      FreeMem(detailData);
		  end  //if (Result != 0)
		end else
			//SetupDiEnumDeviceInterfaces returned 0,     
			LastDevice := TRUE;

		//If we haven't found the device yet, and haven't tried every available device,
		//try the next one.
		  MemberIndex := MemberIndex + 1;
  until not((MyDeviceDetected = FALSE) and (LastDevice = FALSE));

	if (MyDeviceDetected = FALSE) then
		DisplayData('Device not detected')
	else
		DisplayData('Device detected');

	//Free the memory reserved for hDevInfo by SetupDiClassDevs.
	SetupDiDestroyDeviceInfoList(PDevInfo);
	DisplayLastError('SetupDiDestroyDeviceInfoList');

	Result := MyDeviceDetected;
end;

procedure TForm1.GetDeviceCapabilities();
var
  PreparsedData: PHIDPPreparsedData;
  Buffer: array [0..255] of Char;
//  lpBuf: PWideChar;
//  BufLen: Integer;
begin
	//API function: HidD_GetPreparsedData
	//   THIDPPreparsedData,  
  //    HID-.

	HidD_GetPreparsedData(DeviceHandle,PreparsedData);
	DisplayLastError('HidD_GetPreparsedData: ');

	//API function: HidP_GetCaps
	//   

	HidP_GetCaps(PreparsedData, Capabilities);
	DisplayLastError('HidP_GetCaps: ');

	//Display the capabilities

	DisplayData(Format('%s%X', ['Usage Page: ', Capabilities.UsagePage]));
	DisplayData(Format('%s%d', ['Input Report Byte Length: ', Capabilities.InputReportByteLength]));
	DisplayData(Format('%s%d', ['Output Report Byte Length: ', Capabilities.OutputReportByteLength]));
	DisplayData(Format('%s%d', ['Feature Report Byte Length: ', Capabilities.FeatureReportByteLength]));
	DisplayData(Format('%s%d', ['Number of Link Collection Nodes: ', Capabilities.NumberLinkCollectionNodes]));
	DisplayData(Format('%s%d', ['Number of Input Button Caps: ', Capabilities.NumberInputButtonCaps]));
	DisplayData(Format('%s%d', ['Number of InputValue Caps: ', Capabilities.NumberInputValueCaps]));
	DisplayData(Format('%s%d', ['Number of InputData Indices: ', Capabilities.NumberInputDataIndices]));
	DisplayData(Format('%s%d', ['Number of Output Button Caps: ', Capabilities.NumberOutputButtonCaps]));
	DisplayData(Format('%s%d', ['Number of Output Value Caps: ', Capabilities.NumberOutputValueCaps]));
	DisplayData(Format('%s%d', ['Number of Output Data Indices: ', Capabilities.NumberOutputDataIndices]));
	DisplayData(Format('%s%d', ['Number of Feature Button Caps: ', Capabilities.NumberFeatureButtonCaps]));
	DisplayData(Format('%s%d', ['Number of Feature Value Caps: ', Capabilities.NumberFeatureValueCaps]));
	DisplayData(Format('%s%d', ['Number of Feature Data Indices: ', Capabilities.NumberFeatureDataIndices]));
	//   HidD_GetPreparsedData
	HidD_FreePreparsedData(PreparsedData);
	DisplayLastError('HidD_FreePreparsedData: ') ;
end;

procedure TForm1.PrepareForOverlappedTransfer();
begin
	//Get another handle to the device for the overlapped ReadFiles.
	ReadHandle := CreateFile(detailData.DevicePath,
		                      GENERIC_READ or GENERIC_WRITE,
		                      FILE_SHARE_READ or FILE_SHARE_WRITE,
                          nil,
		                      OPEN_EXISTING,
		                      FILE_FLAG_OVERLAPPED,
		                      0);
  if ReadHandle <> INVALID_HANDLE_VALUE then
    DisplayData('Serial number: ' + GetSerialNumber(ReadHandle));
	DisplayLastError('CreateFile (ReadHandle): ');

	//Get an event object for the overlapped structure.

	if (hEventObject = 0) then
	begin
		hEventObject := CreateEvent(nil,TRUE,TRUE,'');
	  DisplayLastError('CreateEvent: ') ;

	  //Set the members of the overlapped structure.
	  HIDOverlapped.hEvent := hEventObject;
	  HIDOverlapped.Offset := 0;
	  HIDOverlapped.OffsetHigh := 0;
	end;
end;

procedure TForm1.ReadReport();
var
	ByteToDisplay: String;
	MessageToDisplay: String;
	Index: DWORD;
  Resultat: LongBool;
begin
	//Read a report from the device.
 	ByteToDisplay:= '';
	MessageToDisplay:= '';

	Resultat := ReadFile
		(ReadHandle,
		InputReport,
		Capabilities.InputReportByteLength,
		NumberOfBytesRead,
		@HIDOverlapped);
 
	DisplayLastError('ReadFile: ') ;

	Index := WaitForSingleObject
		(hEventObject, 
		6000);
	DisplayLastError('WaitForSingleObject: ') ;

	case (Index) of
    WAIT_OBJECT_0:
      DisplayData(Format('%s', ['ReadFile Completed']));
    WAIT_TIMEOUT:
    begin
		  DisplayData(Format('%s', ['ReadFile timeout']));
		  //Cancel the Read operation.

		  Resultat := CancelIo(ReadHandle);
		
		  //A timeout may mean that the device has been removed.
		  //Close the device handles and set DeviceDetected = False
		  //so the next access attempt will search for the device.
		  CloseHandle(ReadHandle);
		  CloseHandle(DeviceHandle);
		  DisplayData('Can''t read from device');
		  DeviceDetected := FALSE;
		end
	  else
		  DisplayData(Format('%s', ['Undefined error']));
	end;

	ResetEvent(hEventObject);

	//Display the report data.
	DisplayInputReport();
end;

procedure TFOrm1.WriteReport();
var
	BytesWritten: DWORD;
	Index: DWORD;
	OutputReport: array [0..255] of Byte;
	Resultat: LongBool;
	strBytesWritten: String;
begin
  BytesWritten:= 0;
	Index:=0;
  strBytesWritten := '';

	//Can set the other report values here, or get them from the combo boxes.
	OutputReport[0] := StrToInt(cbReportId.Text);
	OutputReport[1] := StrToInt(cbByte0.Text);
	OutputReport[2] := StrToInt(cbByte1.Text);

	Resultat := WriteFile
		(DeviceHandle,
		OutputReport,
		Capabilities.OutputReportByteLength,
		BytesWritten,
		nil);

	if (Resultat = FALSE) then
	begin
		//The WriteFile failed, so close the handle, display a message,
		//and set DeviceDetected to FALSE so the next attempt will look for the device.
    if (DeviceHandle <> 0) then
		  CloseHandle(DeviceHandle);
    if (ReadHandle <> 0) then
		  CloseHandle(ReadHandle);
		DisplayData('Can''t write to device');
		DeviceDetected := FALSE;
	end;
	
	//Display the result of the API call and the report bytes.
	DisplayLastError('WriteFile: ');
	DisplayData(Format('%s%d', ['Bytes Written: ', BytesWritten]));
end;

procedure TForm1.DisplayInputReport();
Var
	ByteNumber: DWORD;
	ReceivedByte: Byte;
begin
	//Display the received data in the log and the Bytes Received List boxes.
	//Start at the top of the List Box.
	lbBytesReceived.Clear;
	
	//Step through the received bytes and display each.
  ByteNumber := 0;
  while(ByteNumber < Capabilities.InputReportByteLength) do
	begin
		//Get a byte.
		ReceivedByte := InputReport[ByteNumber];
		//Display it.
		DisplayReceivedData(ReceivedByte);
    inc(ByteNumber);
	end;
end;

procedure TForm1.DisplayCurrentTime();
var
   today : TDateTime;
begin
   today := Now;
   DisplayData(TimeToStr(today) + ',' + DateToStr(today));
end;

procedure TForm1.DisplayData(cstrDataToDisplay: String);
begin
	//Display data in the log List Box
	lbLog.Items.Add(cstrDataToDisplay);
  lbLog.Perform(LB_SETTOPINDEX, Pred(lbLog.Items.Count), 0);
end;

procedure TForm1.DisplayLastError(Operation: String);
const
  NeutralLang : word = (SUBLANG_DEFAULT shl 10) or LANG_NEUTRAL;
var
	lpMsgBuf: PChar;//array[0..1023] of char;
	Index: DWORD;
  strLastError: String;
begin
  Index:= 0;
  strLastError:= '';
  //Display a message and the last error in the log List Box.
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER or
		FORMAT_MESSAGE_FROM_SYSTEM or
		FORMAT_MESSAGE_IGNORE_INSERTS,
		nil,
		GetLastError(),
		0,
		@lpMsgBuf,
		0,
		nil
	);

	//Display the last error.
	strLastError := Operation + lpMsgBuf;
	//Trim CR/LF from the error message.
	Index := lbLog.Items.Add(TrimRight(strLastError));
  lbLog.Perform(LB_SETTOPINDEX, Pred(lbLog.Items.Count), 0);
end;

procedure TForm1.DisplayReceivedData(ReceivedByte: Byte);
var
  strByteRead: String;
begin
	//Convert the value to a 2-character Cstring.
	strByteRead := Format('%03D', [ReceivedByte]);
//	strByteRead := RightStr(strByteRead, 2);
	//Display the value in the Bytes Received List Box.
	lbBytesReceived.Items.Add(strByteRead);
  lbLog.Perform(LB_SETTOPINDEX, Pred(lbLog.Items.Count), 0);
end;

procedure TForm1.btSearchClick(Sender: TObject);
begin
  lbLog.Items.Clear;
end;

procedure TForm1.btTransmitClick(Sender: TObject);
begin
  WriteReport();
end;

procedure TForm1.btReceiverClick(Sender: TObject);
begin
  ReadReport();
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  DisplayData('***HID Test Report***');
  DisplayCurrentTime();

		//If the device hasn't been detected already, look for it.
//		if (DeviceDetected=FALSE) then
  DeviceDetected := FALSE;
  DeviceDetected := FindTheHID();
end;

function TForm1.GetSerialNumber(HidReadHandle: Cardinal): WideString;
var
  I: Integer;
  Len: Integer;
  IDs:    array [0..253] of WORD;
  Buffer: array [0..253] of WideChar;
  SerialNumber: WideString;
begin
  if HidReadHandle <> INVALID_HANDLE_VALUE then
  begin
    FillChar(Buffer, SizeOf(Buffer), #0);
    if HidD_GetSerialNumberString(HidReadHandle, Buffer, SizeOf(Buffer)) then
      begin
        // calculate length of StringDescriptor 0
        FillChar(IDs, SizeOf(IDs), $FF);
        Len := 0;
        HidD_GetIndexedString(HidReadHandle, 0, PWideChar(@IDs), SizeOf(IDs));
        for I := High(IDs) downto 0 do
          if IDs[I] <> $FFFF then
          begin
            if IDs[I] = 0 then
              Len := I
            else
              Len := I+1;
            Break;
          end;
        // compensate for buggy function
        for I := 0 to Len - 1 do
          if IDs[I] <> WORD(Buffer[I]) then
          begin
            SerialNumber := Buffer;
            Break;
          end;
      end;
    end;
  Result := SerialNumber;
end;

procedure TForm1.cbReportIdChange(Sender: TObject);
var
 Str, str1: String;
begin
  case (cbReportId.ItemIndex+1) of
    1:begin
        cbByte0.Items.Text := '0'#13#10; cbByte0.ItemIndex := 0;
        cbByte1.Items.Text := '0'#13#10; end;
    2:begin
        cbByte0.Items.Text := '0'#13#10'1'#13#10'2'#13#10'4'#13#10'8'#13#10'16'#13#10'32'#13#10'64'#13#10'128'#13#10;
        cbByte1.Items.Text := '0'#13#10'1'#13#10'2'#13#10'4'#13#10'8'#13#10'16'#13#10'32'#13#10'64'#13#10'128'#13#10; end;
    3:begin
        cbByte0.Items.Text := '0'#13#10'1'#13#10'2'#13#10'4'#13#10'8'#13#10'16'#13#10'32'#13#10'64'#13#10'128'#13#10;
        cbByte1.Items.Text := '0'#13#10'1'#13#10'2'#13#10'4'#13#10'8'#13#10'16'#13#10'32'#13#10'64'#13#10'128'#13#10; end;
    4:begin
        cbByte0.Items.Text := '0'#13#10;
        cbByte1.Items.Text := '77'#13#10; end;
    5:begin
        cbByte0.Items.Text := '0'#13#10;
        cbByte1.Items.Text := '0'#13#10; end;
    6:begin
        cbByte0.Items.Text := '0'#13#10;
        cbByte1.Items.Text := '7'#13#10; end;
    7:begin
        cbByte0.Items.Text := '0'#13#10;
        cbByte1.Items.Text := '18'#13#10; end;
    8:begin
        cbByte0.Items.Text := '0'#13#10;
        cbByte1.Items.Text := '12'#13#10; end;
  else
    cbByte0.Items.Text := '0'#13#10;
    cbByte1.Items.Text := '0'#13#10;
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  ReadReport();
end;

procedure TForm1.cbAutoClick(Sender: TObject);
begin
  if (cbAuto.Checked = true) then
    Timer1.Enabled := true
  else
    Timer1.Enabled := false;
end;

end.
